@@ -60,6 +60,7 @@ func TestProgressEmitter(t *testing.T) {
 
 	p := NewProgressEmitter(c)
 	go p.Serve()
+	p.interval = 0
 
 	expectTimeout(w, t)
 
@@ -106,7 +107,7 @@ func TestSendDownloadProgressMessages(t *testing.T) {
 		TempIndexMinBlocks:      10,
 	})
 
-	fc := &FakeConnection{}
+	fc := &fakeConnection{}
 
 	p := NewProgressEmitter(c)
 	p.temporaryIndexSubscribe(fc, []string{"folder", "folder2"})
