@@ -273,7 +273,7 @@ func BenchmarkJobQueuePushPopDone10k(b *testing.B) {
 		for _, f := range files {
 			q.Push(f.Name, 0, time.Time{})
 		}
-		for _ = range files {
+		for range files {
 			n, _ := q.Pop()
 			q.Done(n)
 		}
