@@ -22,10 +22,7 @@ func SyncFile(path string) error {
 	}
 	defer fd.Close()
 	// MacOS and Windows do not flush the disk cache
-	if err := fd.Sync(); err != nil {
-		return err
-	}
-	return nil
+	return fd.Sync()
 }
 
 func SyncDir(path string) error {
